import os
import pandas
import selenium
from time import sleep
from .helper import Helper
from selenium import webdriver
import geckodriver_autoinstaller
geckodriver_autoinstaller.install()


class Contributing:

    def editors(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total', 'page-type-content':'page_type~content', 'page-type-non-content':'page_type~non-content', 'page-type-all':'page_type~content*non-content',
                       'editor-type-user':'editor_type~user', 'editor-type-name-bot':'editor_type~name-bot', 'editor-type-anonymous':'editor_type~anonymous', 'editor-type-group-bot':
                       'editor_type~group-bot', 'editor-type-all':'editor_type~anonymous*group-bot*name-bot*user', 'activity-level-1-to-4-edits':'activity_level~1..4-edits',
                       'activity-level-5-to-24-edits':'activity_level~5..24-edits', 'activity-level-25-to-99-edits':'activity_level~25..99-edits',
                        'activity-level-100-or-more-edits':'activity_level~100..-edits', 'activity-level-all':'activity_level~1..4-edits*5..24-edits*25..99-edits*100..-edits'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/editors/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--editors--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def active_editors(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total', 'page-type-content':'page_type~content', 'page-type-non-content':'page_type~non-content', 'page-type-all':'page_type~content*non-content'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/active-editors/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--active-editors--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def edits(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total', 'page-type-content':'page_type~content', 'page-type-non-content':'page_type~non-content', 'page-type-all':'page_type~content*non-content',
                       'editor-type-user':'editor_type~user', 'editor-type-name-bot':'editor_type~name-bot', 'editor-type-anonymous':'editor_type~anonymous', 'editor-type-group-bot':
                       'editor_type~group-bot', 'editor-type-all':'editor_type~anonymous*group-bot*name-bot*user'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/edits/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--edits--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def user_edits(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total', 'page-type-content':'page_type~content', 'page-type-non-content':'page_type~non-content', 'page-type-all':'page_type~content*non-content'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/user-edits/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--user-edits--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def new_pages(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total', 'page-type-content':'page_type~content', 'page-type-non-content':'page_type~non-content', 'page-type-all':'page_type~content*non-content',
                       'editor-type-user':'editor_type~user', 'editor-type-name-bot':'editor_type~name-bot', 'editor-type-anonymous':'editor_type~anonymous', 'editor-type-group-bot':
                       'editor_type~group-bot', 'editor-type-all':'editor_type~anonymous*group-bot*name-bot*user'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/new-pages/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--new-pages--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def new_registered_users(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'all-years':'all', 'one-year':'1-year', 'two-years':'2-year', 'three-months':'3-month', 'one-month':'1-month'}

            filters = {'no-filter':'~total'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/new-registered-users/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--new-registered-users--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def top_editors(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'last-month':'last-month'}

            # if period not in periods:
            #     periods.update({period:period})

            filters = {'no-filter':'~total', 'page-type-content':'(page_type)~content', 'page-type-non-content':'(page_type)~non-content', 'page-type-all':'(page_type)~content*non-content',
                       'editor-type-user':'(editor_type)~user', 'editor-type-name-bot':'(editor_type)~name-bot', 'editor-type-anonymous':'(editor_type)~anonymous', 'editor-type-group-bot':
                       '(editor_type)~group-bot', 'editor-type-all':'(editor_type)~anonymous*group-bot*name-bot*user'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/top-editors/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--top-editors--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def top_edited_pages(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'last-month':'last-month'}

            # if period not in periods:
            #     periods.update({period:period})

            filters = {'no-filter':'~total', 'page-type-content':'(page_type)~content', 'page-type-non-content':'(page_type)~non-content', 'page-type-all':'(page_type)~content*non-content',
                       'editor-type-user':'(editor_type)~user', 'editor-type-name-bot':'(editor_type)~name-bot', 'editor-type-anonymous':'(editor_type)~anonymous', 'editor-type-group-bot':
                       '(editor_type)~group-bot', 'editor-type-all':'(editor_type)~anonymous*group-bot*name-bot*user'}

            intervals = {'daily':'daily', 'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/top-edited-pages/full|table|'
            parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--top-edited-pages--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")


    def active_editors_by_country(wiki, period, filter, interval):
        try:

            try:
                wikis = Helper.get_Wikis_Codes()
            except NameError:
                print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                exit()

            periods = {'last-month':'last-month'}

            # if period not in periods:
            #     periods.update({period:period})

            filters = {'activity-level-5-to-99-edits':'(activity-level)~5..99-edits', 'activity-level-100-or-more-edits':'(activity-level)~100..-edits'}

            intervals = {'monthly':'monthly'}

            options = webdriver.FirefoxOptions()
            options.add_argument("--headless")
            profile = webdriver.FirefoxProfile()
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.manager.showWhenStarting", False)
            profile.set_preference("browser.download.dir", f"{os.getcwd()}")
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/octet-stream")
            driver = webdriver.Firefox(options=options, firefox_profile=profile, executable_path='geckodriver', service_log_path=os.devnull)

            base_url = f'https://stats.wikimedia.org/#/{wiki}.wikipedia.org/contributing/active-editors-by-country/full|table|'
            if filter=='activity-level-5-to-99-edits':
                parameters = f'{periods[period]}|(activity-level)~5..99-edits|{intervals[interval]}'
            elif filter=='activity-level-100-or-more-edits':
                parameters = f'{periods[period]}|(activity-level)~100..-edits|{intervals[interval]}'
            else:
                parameters = f'{periods[period]}|{filters[filter]}|{intervals[interval]}'
            request_url = "".join([base_url, parameters])

            driver.implicitly_wait(5)
            driver.get(request_url)
            html = driver.page_source

            if intervals[interval]=='daily':
                sleep(10*2)
            if intervals[interval]=='monthly':
                sleep(10)

            csvFile = f"{(wikis[wiki]).lower()}--active-editors-by-country--{filter}--{period}--{interval}.csv"
            csvFile = csvFile.replace(' ','-')
            driver.find_element_by_class_name("ui.icon.button.tooltipped.tooltipped-n").click()
            sleep(3) ; os.rename("undefined.csv", csvFile)

            print(f"## Downloaded `{csvFile}` successfully :-)\n")
            print(f"** Quick glance at `{csvFile}` file:")
            readCSVfile = pandas.read_csv(csvFile)
            print(readCSVfile.to_string(max_rows=5), '\n')

            driver.close()
            driver.quit()

        except KeyboardInterrupt:
            print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
            exit()

        except KeyError:
            if wiki not in wikis:
                print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
            elif period not in periods:
                print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
            elif filter not in filters:
                print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            elif interval not in intervals:
                print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
            else: 
                print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
        
        except selenium.common.exceptions.WebDriverException:
            print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
            print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")

        except selenium.common.exceptions.NoSuchElementException:
            print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")

        except:
            if 'Loading metric...' in html:
                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
                print(request_url, '\n')
            elif 'There is no data available for this date range on this project' in html:
                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
            else: 
                print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")



import sys
import rich
from .helper import Helper
from .content import Content
from .reading import Reading
from .contributing import Contributing
from .custom_argparse import ArgumentParser


def list_Wikis_Codes():
    wikis = Helper.get_Wikis_Codes()
    print(f"## List of `{len(wikis)}` Wikipedias and its languages codes:\n")
    for code, wiki in wikis.items():
        print(f"{wiki}:`{code}`", end=', ')
    print('\n') ; exit()


def main():

    rich.print('''
            [dodger_blue2]▌│║▌║▌█║▌║║▌│║▌│║▌║▌█║[/][spring_green4]▌█║▌║│▌║│▌║│█║│▌[/][dodger_blue2]║▌│█║▌│▌║│▌║║▌║▌█║▌║│▌[/]
            [dodger_blue2]▌│║▌║▌█║▌║║▌│║▌│║▌║▌█║[/][red]WIKISTATS-TO-CSV[/][dodger_blue2]║▌│█║▌│▌║│▌║║▌║▌█║▌║│▌[/]
            [dodger_blue2]▌│║▌║▌█║▌║║▌│║▌│║▌║▌█║[/][spring_green4]▌█║▌║│▌║│▌║│█║│▌[/][dodger_blue2]║▌│█║▌│▌║│▌║║▌║▌█║▌║│▌[/]
            ''')

    try:
        if sys.argv[1]=='-lw' or sys.argv[1]=='--list-wikis':
            list_Wikis_Codes()
    except IndexError:
        pass

    parser = ArgumentParser(add_help=False)
    parser.add_argument('-h', '--help', action='help', help='Show Help Message')
    parser.add_argument('-w', '--wiki', type=str, required=True, metavar='', help='Wikipedia Codes')
    parser.add_argument('-m', '--metric', type=str, required=True, metavar='', help='Wikipedia Metrics')
    parser.add_argument('-q', '--query', type=str, required=True, metavar='', help='Wikipedia Queries')
    parser.add_argument('-p', '--period', type=str, required=False, default='all-years', metavar='', help='Time Periods')
    parser.add_argument('-f', '--filter', type=str, required=False, default='no-filter', metavar='', help='Metrics Filters')
    parser.add_argument('-i', '--interval', type=str, required=False, default='monthly', metavar='', help='Time Intervals')
    parser.add_argument('-lw', '--list-wikis', type=str, required=False, metavar='', help='List Wikipedias')
    
    try:
        args = parser.parse_args()
    
        if args.wiki!=None:

            if args.metric=='content':
                if args.query=='pages-to-date':
                    Content.pages_to_date(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='edited-pages':
                    Content.edited_pages(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='absolute-bytes-difference':
                    Content.absolute_bytes_difference(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='net-bytes-difference':
                    Content.net_bytes_difference(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='total-media-requests':
                    Content.total_media_requests(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='top-media-requests':
                    if args.period=='last-month':
                        Content.top_media_requests(args.wiki, args.period, args.filter, args.interval)
                    elif args.period!=None and args.period!='all-years':
                        Content.top_media_requests(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query `top-media-requests` has only one pre-defined time period: 'last-month'.\n")

                else:
                    print(f"\u001b[91mError:\u001b[0m wikistats2csv metric `content` does not support this query: '{args.query}'.\n")

            elif args.metric=='reading':
                if args.query=='total-page-views':
                    Reading.total_page_views(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='legacy-page-views':
                    Reading.legacy_page_views(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='page-views-by-country':
                    if args.period=='last-month':
                        Reading.page_views_by_country(args.wiki, args.period, args.filter, args.interval)
                    elif args.period!=None and args.period!='all-years':
                        Reading.page_views_by_country(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query `page-views-by-country` has one pre-defined time period: 'last-month'.\n")

                elif args.query=='unique-devices':
                    Reading.unique_devices(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='top-viewed-articles':
                    if args.period=='last-month':
                        Reading.top_viewed_articles(args.wiki, args.period, args.filter, args.interval)
                    elif args.period!=None and args.period!='all-years':
                        Reading.top_viewed_articles(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query `top-viewed-articles` has one pre-defined time period: 'last-month'.\n")

                else:
                    print(f"\u001b[91mError:\u001b[0m wikistats2csv metric `reading` does not support this query: '{args.query}'.\n")

            elif args.metric=='contributing':
                if args.query=='editors':
                    Contributing.editors(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='active-editors':
                    Contributing.active_editors(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='edits':
                    Contributing.edits(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='user-edits':
                    Contributing.user_edits(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='new-pages':
                    Contributing.new_pages(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='new-registered-users':
                    Contributing.new_registered_users(args.wiki, args.period, args.filter, args.interval)

                elif args.query=='top-editors':
                    if args.period=='last-month':
                        Contributing.top_editors(args.wiki, args.period, args.filter, args.interval)
                    elif args.period!=None and args.period!='all-years':
                        Contributing.top_editors(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query  `top-editors`  has  only  one  pre-defined  time period: 'last-month'.\n")

                elif args.query=='top-edited-pages':
                    if args.period=='last-month':
                        Contributing.top_edited_pages(args.wiki, args.period, args.filter, args.interval)
                    elif args.period!=None and args.period!='all-years':
                        Contributing.top_edited_pages(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query  `top-edited-pages`  has  only  one  pre-defined  time period: 'last-month'.\n")

                elif args.query=='active-editors-by-country':
                    if args.period=='last-month':
                        if args.filter!='no-filter':
                            Contributing.active_editors_by_country(args.wiki, args.period, args.filter, args.interval)
                        else:
                            print("\u001b[91mError:\u001b[0m query  `active-editors-by-country`  has  only  two  pre-defined  filters:")
                            print("       Use: 'activity-level-5-to-99-edits' or 'activity-level-100-or-more-edits'.\n")
                    elif args.period!=None and args.period!='all-years':
                        Contributing.active_editors_by_country(args.wiki, args.period, args.filter, args.interval)
                    else:
                        print("\u001b[91mError:\u001b[0m query `active-editors-by-country` has one pre-defined time period: 'last-month'.\n")

                else:
                    print(f"\u001b[91mError:\u001b[0m wikistats2csv metric `contributing` does not support this query: '{args.query}'.\n")
            
            else:
                print(f"\u001b[91mError:\u001b[0m wikistats2csv does not support this metric: '{args.metric}'.\n")

    except SystemExit:
        print('')


if __name__ == "__main__":
    main()


import ssl
import pandas


# To Override the SSL Error of '[SSL: CERTIFICATE_VERIFY_FAILED]'. 
ssl._create_default_https_context = ssl._create_unverified_context


class Helper:
    def get_Wikis_Codes():
        url = r'https://en.wikipedia.org/wiki/List_of_Wikipedias'
        tables = pandas.read_html(url)
        wikis_codes = tables[2] 
        wikis_codes = wikis_codes[['Wiki', 'Language']]
        wikis_codes = wikis_codes.set_index('Wiki').to_dict()['Language']
        return wikis_codes


import codecs
from setuptools import setup
from setuptools import find_packages


with codecs.open('README.md', 'r', 'utf8') as reader:
    long_description = reader.read()


setup(
    name = 'wikistats2csv',
    version = '0.1.2',
    license = 'MIT',
    author = 'Saied Alshahrani',
    author_email = 's3ed.fala7@gmail.com',
    packages = find_packages(),
    description='Wikistats-to-CSV downloads Wikipedia Statistics in CSV format for a given Wikipedia.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/SaiedAlshahrani/Wikistats-to-CSV',
    keywords = ['wikipedia','statistics', 'wikimedia', 'csv'],
    install_requires = ['lxml==4.9.1', 'rich==12.5.1','pandas==1.4.3','selenium==3.141.0', 'geckodriver-autoinstaller==0.1.0'],
    entry_points = {'console_scripts':['wikistats2csv = wikistats2csv.wikistats2csv:main']},
    classifiers = ['Programming Language :: Python :: 3', 'License :: OSI Approved :: MIT License', 'Operating System :: OS Independent']
    )
